/**********************************************************************
 Parser for Simple programs
 **********************************************************************/

// The types of the tokens and nonterminals
%union {
    Token               TOKENval;   // type of all tokens
    Body *              BodyVal;
    Joint *             JointVal;
    Exp *               ExpVal;
    ConstExp *          ConstExpVal;
    Body::container *   BodyList;
    Joint::container *  JointList;
    Exp::container *    ExpList;
    V2fproxy            V2fVal;
};

%token <TOKENval> INTEGER
%token <TOKENval> BOOLEAN
%token <TOKENval> STRING
%token <TOKENval> CONSTANT
%token <TOKENval> BODIES
%token <TOKENval> BODY
%token <TOKENval> JOINTS
%token <TOKENval> JOINT
%token <TOKENval> CONSTANTJOINT
%token <TOKENval> EXPS
%token <TOKENval> TIME
%token <TOKENval> SIN
%token <TOKENval> COS
%token <TOKENval> LPAREN
%token <TOKENval> RPAREN
%token <TOKENval> PLUS
%token <TOKENval> UNARYMINUS
%token <TOKENval> TIMES
%token <TOKENval> GREATER
%token <TOKENval> LESS
%token <TOKENval> COMMA

%type <BodyList> configfile
%type <BodyList> allbodies
%type <BodyVal>  body
%type <JointList> alljoints
%type <JointVal> joint
%type <JointVal> normaljoint
%type <JointVal> constantjoint
%type <ExpList> allexps
%type <ExpVal> exp
%type <ExpVal> expint
%type <ConstExpVal> constExp
%type <ExpVal> timeExp
%type <ExpVal> sinExp
%type <ExpVal> cosExp
%type <ExpVal> plusExp
%type <ExpVal> unaryMinusExp
%type <ExpVal> timesExp
%type <ExpVal> greaterExp
%type <ExpVal> lessExp
%type <V2fVal> v2f
 
%start configfile

%{
// Precedences and associativities --------------------------
%}

%%

configfile      : BODIES allbodies JOINTS alljoints EXPS allexps
                  {
                      $$ = $2;
                  }
                ;

allbodies       : allbodies body
                  {
                      $$ = $1;
                      $$->push_back($2);
                  }
                | /* epsilon */
                  {
                      $$ = _bodies;
                      $$->clear();
                  }
                ;

body            : LPAREN BODY v2f v2f CONSTANT RPAREN
                  { 
                      $$ = new Body(V2f($3.x, $3.y), V2f($4.x, $4.y), 
                                    $5.value);
                  }
		;

alljoints       : alljoints joint
                  {
                      $$ = $1;
                      $$->push_back($2);
                  }
                | alljoints constantjoint
                  {
                      $$ = $1;
                      $$->push_back($2);
                  }
                | /* epsilon */
                  {
                      $$ = _joints;
                      $$->clear();
                  }
                ;

joint           : LPAREN normaljoint RPAREN
                  {
                      $$ = $2;
                  }
                | LPAREN constantjoint RPAREN
                  {
                      $$ = $2;
                  }
		;

normaljoint     : JOINT LPAREN CONSTANT COMMA CONSTANT RPAREN LPAREN 
                  CONSTANT COMMA CONSTANT RPAREN CONSTANT
                  {
                      $$ = new Joint((int) $3.value, $5.value, 
                                     (int) $8.value, $10.value,
                                     $12.value, 0);
                  }
		;

constantjoint   : CONSTANTJOINT LPAREN CONSTANT COMMA CONSTANT RPAREN LPAREN
                  CONSTANT COMMA CONSTANT RPAREN CONSTANT
                  {
                      $$ = new ConstantJoint((int) $3.value, $5.value, 
                                             (int) $8.value, $10.value,
                                             $12.value);
                  }
		;

allexps         : allexps exp
                  {
                      $$ = $1;
                      $$->push_back($2);
                  }
                | /* epsilon */
                  {
                      $$ = _exps;
                      $$->clear();
                  }
                ;

exp             : LPAREN expint RPAREN
                  { $$ = $2; }
		;

expint          : constExp
                  { $$ = $1; }
		;
                | timeExp
                  { $$ = $1; }
		;
                | unaryMinusExp
                  { $$ = $1; }
		;
                | sinExp
                  { $$ = $1; }
		;
                | cosExp
                  { $$ = $1; }
		;
                | plusExp
                  { $$ = $1; }
		;
                | timesExp
                  { $$ = $1; }
		;
                | greaterExp
                  { $$ = $1; }
		;
                | lessExp
                  { $$ = $1; }
		;

constExp     : CONSTANT
                  { $$ = new ConstExp($1.value); }
		;

timeExp         : TIME
                  { $$ = new TimeExp(); }
		;

unaryMinusExp   : UNARYMINUS exp
                  { $$ = new UnaryMinusExp($2); }
		;

sinExp          : SIN exp
                  { $$ = new SinExp($2); }
		;

cosExp          : COS exp
                  { $$ = new CosExp($2); }
		;

plusExp         : PLUS exp exp
                  { $$ = new PlusExp($2, $3); }
		;

timesExp        : TIMES exp exp
                  { $$ = new TimesExp($2, $3); }
		;

greaterExp      : GREATER LPAREN constExp RPAREN exp exp
                  { $$ = new GreaterExp($3, $5, $6); }
		;

lessExp         : LESS LPAREN constExp RPAREN exp exp
                  { $$ = new LessExp($3, $5, $6); }
		;

v2f             : LPAREN CONSTANT COMMA CONSTANT RPAREN
                  { $$.x = $2.value; $$.y = $4.value; }
		;

%%

